@@ -1,5 +1,5 @@
 package App::rshasum;
-$App::rshasum::VERSION = '0.0.2';
+$App::rshasum::VERSION = '0.2.0';
 use strict;
 use warnings;
 
@@ -8,6 +8,37 @@ use Digest             ();
 
 use Getopt::Long qw/ GetOptions /;
 
+sub _worker
+{
+    my ( $self, $args ) = @_;
+
+    my $digest    = $args->{digest};
+    my $output_cb = $args->{output_cb};
+
+    my $t = Digest->new($digest);
+
+    my $ff = File::Find::Object->new( {}, "." );
+    while ( my $r = $ff->next_obj )
+    {
+        if ( $r->is_file )
+        {
+            my $d    = Digest->new($digest);
+            my $path = join '/', @{ $r->full_components };
+            open my $fh, '<', $r->path;
+            binmode $fh;
+            $d->addfile($fh);
+            close $fh;
+            my $s = $d->hexdigest . '  ' . $path . "\n";
+            $output_cb->( { str => $s } );
+            $t->add($s);
+        }
+    }
+    my $s = $t->hexdigest . '  ' . "-" . "\n";
+    $output_cb->( { str => $s } );
+
+    return;
+}
+
 sub run
 {
     my $digest;
@@ -17,27 +48,12 @@ sub run
     {
         die "Please give a --digest=[digest] argument.";
     }
-
-    my $t = Digest->new($digest);
-
-    my $ff = File::Find::Object->new( {}, "." );
-    while ( my $r = $ff->next_obj )
-    {
-        if ( $r->is_file )
+    return shift()->_worker(
         {
-            my $d    = Digest->new($digest);
-            my $path = $r->path;
-            open my $fh, '<', $path;
-            binmode $fh;
-            $d->addfile($fh);
-            close $fh;
-            my $s = $d->hexdigest . '  ' . $path . "\n";
-            print $s;
-            $t->add($s);
+            digest    => $digest,
+            output_cb => sub { print shift()->{str}; }
         }
-    }
-    my $s = $t->hexdigest . '  ' . "-" . "\n";
-    print $s;
+    );
 }
 
 1;
@@ -54,7 +70,7 @@ App::rshasum
 
 =head1 VERSION
 
-version 0.0.2
+version 0.2.0
 
 =head1 SYNOPSIS
 
@@ -66,7 +82,7 @@ App::rshasum - recursive shasum.
 
 =head1 VERSION
 
-version 0.0.2
+version 0.2.0
 
 =head1 METHODS
 
